package model

import (
	"encoding/json"
	"time"

	"go.mongodb.org/mongo-driver/bson/primitive"
)

type EffectType struct {
	ID             primitive.ObjectID `json:"id,omitempty" bson:"_id,omitempty"`
	ForeignID      string             `json:"foreignId" bson:"foreignId,omitempty"`
	CreatedAt      time.Time          `json:"createdAt" bson:"createdAt,omitempty"`
	LastModifiedAt time.Time          `json:"lastModifiedAt" bson:"lastModifiedAt,omitempty"`
	TypeName       string             `json:"typeName" bson:"typeName,omitempty"`
	Nls            map[string]string  `json:"nls" bson:"nls,omitempty"`
	TypeImage      string             `json:"typeImage" bson:"typeImage,omitempty"`
}

func NewEffectType() EffectType {
	effectType := EffectType{
		Nls: make(map[string]string),
	}
	return effectType
}

//UnmarshalJSON unmarschal function
func (s *EffectType) UnmarshalJSON(data []byte) error {
	var dat map[string]interface{}
	if err := json.Unmarshal(data, &dat); err != nil {
		return err
	}
	if dat["id"] != nil {
		id, _ := primitive.ObjectIDFromHex(dat["id"].(string))
		s.ID = id
	}
	if dat["foreignId"] != nil {
		s.ForeignID = dat["foreignId"].(string)
	}
	if dat["createdAt"] != nil {
		switch v := dat["createdAt"].(type) {
		case string:
			layout := "2006-01-02T15:04:05.000Z"
			s.CreatedAt, _ = time.Parse(layout, v)
		case float64:
			s.CreatedAt = time.Unix(0, int64(v)*int64(time.Millisecond))
		}
	}
	if dat["lastModifiedAt"] != nil {
		switch v := dat["lastModifiedAt"].(type) {
		case string:
			layout := "2006-01-02T15:04:05.000Z"
			s.LastModifiedAt, _ = time.Parse(layout, v)
		case float64:
			s.LastModifiedAt = time.Unix(0, int64(v)*int64(time.Millisecond))
		}
	}
	if dat["typeName"] != nil {
		s.TypeName = dat["typeName"].(string)
	}
	if dat["nls"] != nil {
		switch v := dat["nls"].(type) {
		case []interface{}:
			values := v
			s.Nls = make(map[string]string)
			for _, d := range values {
				s.Nls["de"] = d.(string)
			}
		case map[string]interface{}:
			values := v
			s.Nls = make(map[string]string)
			for k, d := range values {
				s.Nls[k] = d.(string)
			}
		}
	}
	if dat["typeImage"] != nil {
		s.TypeImage = dat["typeImage"].(string)
	}
	return nil
}
