package model

import "go.mongodb.org/mongo-driver/bson/primitive"

//Tag a single tag
type Tag struct {
	ID    primitive.ObjectID `json:"-" bson:"_id,omitempty"`
	Name  string             `json:"name" bson:"name,omitempty"`
	Count int                `json:"count" bson:"count,omitempty"`
}

//Tags tag list object
type Tags struct {
	List []Tag
}

//NewTags creates a new tag list
func NewTags() Tags {
	m := Tags{
		List: make([]Tag, 0),
	}
	return m
}

//Contains checking if the manufacturer name is present in the list of manufacturers
func (m *Tags) Contains(name string) bool {
	for _, a := range m.List {
		if a.Name == name {
			return true
		}
	}
	return false
}

//Add adding a new manufacturer to the list
func (m *Tags) Add(name string, count int) {
	tag, ok := m.Get(name)
	if !ok {
		m.List = append(m.List, Tag{
			Name:  name,
			Count: count,
		})
	} else {
		if tag.Count != count {
			m.Remove(name)
			m.List = append(m.List, Tag{
				Name:  name,
				Count: count,
			})
		}
	}
}

//Remove remove a single tag
func (m *Tags) Remove(name string) {
	index := m.Index(name)
	m.List[index] = m.List[len(m.List)-1]
	m.List = m.List[:len(m.List)-1]
}

//Index getting a tag index
func (m *Tags) Index(name string) int {
	for i, a := range m.List {
		if a.Name == name {
			return i
		}
	}
	return -1
}

//Get getting a tag
func (m *Tags) Get(name string) (Tag, bool) {
	for _, a := range m.List {
		if a.Name == name {
			return a, true
		}
	}
	return Tag{}, false
}

//Cleear clearing the list
func (m *Tags) Clear() {
	m.List = nil
}
